@@ -25,15 +25,15 @@ test_that("save_print_and_play works as expected", {
     dir.create(pdf_deck_dir)
     on.exit(unlink(pdf_deck_dir, recursive=TRUE))
     pdf_deck_filename <- file.path(pdf_deck_dir, "piecepack_deck.pdf")
-    on.exit(unlink(pdf_deck_filename))
+    on.exit(unlink(pdf_deck_filename), add = TRUE)
     pdf_deck_filename_a4 <- file.path(pdf_deck_dir, "piecepack_deck_a4.pdf")
-    on.exit(unlink(pdf_deck_filename_a4))
+    on.exit(unlink(pdf_deck_filename_a4), add = TRUE)
     pdf_deck_filename_a5 <- file.path(pdf_deck_dir, "piecepack_deck_a5.pdf")
-    on.exit(unlink(pdf_deck_filename_a5))
+    on.exit(unlink(pdf_deck_filename_a5), add = TRUE)
     pdf_deck_filename_5s <- file.path(pdf_deck_dir, "piecepack_deck_5s.pdf")
-    on.exit(unlink(pdf_deck_filename_5s))
+    on.exit(unlink(pdf_deck_filename_5s), add = TRUE)
     pdf_deck_filename_4x6 <- file.path(pdf_deck_dir, "piecepack_deck_4x6.pdf")
-    on.exit(unlink(pdf_deck_filename_4x6))
+    on.exit(unlink(pdf_deck_filename_4x6), add = TRUE)
     pdf_deck_filename_bleed <- file.path(pdf_deck_dir, "piecepack_deck_bleed.pdf")
     cfg_5s <- list(suit_text="♥,★,♣,♦,♠,꩜", suit_color="darkred,gold,darkgreen,darkblue,black,grey")
 
@@ -112,7 +112,7 @@ test_that("no regressions in figures", {
         tmpfile <- tempfile(fileext=".svg")
         on.exit(unlink(tmpfile))
         svg(tmpfile)
-        on.exit(dev.off())
+        on.exit(dev.off(), add = TRUE)
         grid.piece(..., default.units="npc")
     }
     # tile back
